import compiler
import Numeric
import numarray
import numarray._numarray as _numarray
import numarray.memory as memory
import numarray._ufunc as _ufunc
import sys
import string
import re
import profile
import hotshot
import hotshot.stats
import timeit

from optparse import OptionParser

parameters = """
a=arange(10**i); j=max(int(10**(i/2.0)),10**3); b=reshape(arange(j*j),(j,j));   c=a.copy(); d=a.copy(); e=transpose(b); f=transpose(b); af=a.astype('d'); cf=a.astype('d'); z=array(0); s='\1\1\1\1'*10**i
"""

param_funcs = ["arange", "array", "reshape", "transpose"]

benchmarks = """
take(a,c)
put(a,c,d)
transpose(b)
putmask(a,c%2,d)
repeat(a,2)
choose(a%2,(a,c))
ravel(b)
nonzero(a)
where(a%2,a,c)
compress(a%2,a)
diagonal(b)
trace(b)
searchsorted(a,a)
sort(a)
argsort(a)
argmax(b)
array(s,typecode='i')
dot(b,b)
clip(a,50,100)
indices((100,100))
swapaxes(b,0,-1)
concatenate((a,a,a))
innerproduct(b,b)
outerproduct(b[0],b[0])
resize(a,(2*10**i,))
a[0]                                   # single element
b[0]                                   # single row
a[1000:2000]                           # 1D slice
b[400:600,400:600]                     # 2D slice
arange(10.**i)
identity(i+2)
add(a,c)
add(a,c,d)
a+c
cos(a)
add.reduce(a)
add.reduce(b,0)
add.reduce(b,-1)
add.accumulate(a)
add.accumulate(b,0)
add.accumulate(b,1)
"""

parameters = parameters.strip()

def run(benchmark, *args):
    _setup = "; ".join(args)
    # print _setup
    t = timeit.Timer(benchmark, setup=_setup)
    number = 1
    repeats = 3
    return (min(t.repeat(repeats,number))/number)*10.0**6

def report(line,i,show_bench):
    bmark = line.split(" ")[0]
    function = bmark.split("(")[0]
    function = function.split(".")[0]
    if function == bmark:
        function = "array"

    Numeric_imp  = "from Numeric import "  + ", ".join(param_funcs+[function])
    numarray_imp = "from numarray import " + ", ".join(param_funcs+[function])
    
    m = run(bmark, Numeric_imp,  "i=%d" % i, parameters)
    n = run(bmark, numarray_imp, "i=%d" % i, parameters)

    if m and n:
        try:
            r = n/m
        except:
            r = 0
    elif n:
        r = 0
    else:
        raise RuntimeError("Invalid benchmark")
    if not show_bench:
        bmark = ""
        
    print "%-25s%5d%18d%18d%18.1f" % (bmark, i, n, m, r)

def marks(pattern):
    if pattern != None:
        l = []
        lines = string.split(benchmarks,"\n")
        for i in range(len(lines)):
            if re.match(pattern, lines[i]):
                l.append(lines[i])
    else:
        l = benchmarks.split("\n")
    return l

def benchmark(pattern=None, f=report, powers=[0,5]):
    print "%-25s10**x%18s%18s%18s" % ("benchmark","numarray (usec)","Numeric (usec)","numarray:Numeric")
    for m in marks(pattern):
        if m:
            f(m, powers[0], show_bench=1)
            for p in powers[1:]:
                f(m, p, show_bench=0)

def prof_do(mark, m, verbose=1):
    pr = hotshot.Profile("prof.out", 1, 1)
    # pr.calibrate(100, verbose=1)
    pr.run(mark)
    if verbose:
        print "profile for: ", m
    pr.close()
    pr = hotshot.stats.load("prof.out")
    pr.strip_dirs().sort_stats("time")
    pr.print_stats()

def prof(m, verbose=1):
    params = parameters.split("\n")
    globals().update(numarray.__dict__)
    for p in params[1].split():
        exec p in globals()
    line = m.split(" ")
    mark, comment = line[0], " ".join(line[1:])
    prof_do(mark, m, verbose)

def profall(pattern=None):
    for m in marks(pattern):
        if m:
            print "-"*70
            prof(m)
            prof(m + " (cached)")



def main():

    parser = OptionParser()
    parser.add_option("-p", "--powers", dest="powers",
                      help="powers of 10 to bencmark", metavar="POWERS",
                      default="[0,5]")
    (options, args) = parser.parse_args()

    POWERS = eval(options.powers)

    if len(args) == 0:
        benchmark(powers=POWERS)
    else:
        if sys.argv[1] == "prof":
            for p in sys.argv[1:]:
                profall(p)
        else:
            for b in args:
                benchmark(b, powers=POWERS)

if __name__ == "__main__":
    main()

